﻿using System.Globalization;
using System.Reflection;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Messages;
using Produmex.Foundation.SlimScreen.Interfaces;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.SlimScreen;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using System.Text;

using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;

using System.Data;
using System;


// PmxWorkflowExecutionTypes.HOOK_FLOW
namespace WorkflowScript_AfterSSCCIdentifiedForAdHocMoveHookScript
{
    /// <summary>
    /// This is a hook script, meant to be customized on a "per project" basis
    /// </summary>
    public class WorkflowScript_AfterSSCCIdentifiedForAdHocMoveHookScript : WorkflowInstanceScriptBase
    {
        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        // Input parameters * do not change *
        public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
        public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;
        public ReadOnlyBinder<string> SSCC;

        // Output parameters * do not change *
        public ReadWriteBinder<bool> BackRequested;

		// Sub-flows
		// <none>

		/// <summary>
		/// Initializes a new instance of the <see cref="WorkflowScript_AfterSSCCIdentifiedForAdHocMoveHookScript"/> class.
		/// </summary>
		/// <param name="parent">The parent.</param>
		/// <param name="factory">The factory.</param>
		public WorkflowScript_AfterSSCCIdentifiedForAdHocMoveHookScript( WorkflowInstanceBase parent, WorkflowInstanceFactory factory )
            : base(parent, factory)
        {
        }

        #region WorkflowInstanceScriptBase Members

        protected override void Execute()
        {			
		   Session session = GetScopeParameter("Session") as Session;
		   ISboProviderService sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;

			Message msg = null;
			

			string query = BuildQuery.GetSSCCInformation(SSCC);
			DataSet ds = sboProviderService.RunView(false, null, null, query);

			if (this.GetNumberOfRows(ds) > 0)
			{
				//string qryGroup16 = ds.Tables[0].Rows[0][SboItemTable.Columns.QryGroup16.NAME].ToString();

				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
					this.DefaultCultureInfo, BuildParamCollection(
					"MessageKey", "SSCC: " + SSCC + " , Quantity : " + Convert.ToInt32(ds.Tables[0].Rows[0][PmxInventoryDetailTable.Columns.Quantity.NAME]),
					"ShowButton", true
					));
				msg = WaitForMessage();
			}


		}

		#endregion
		#region Helpers
		private int GetNumberOfRows(DataSet ds)
		{
			if (ds != null && ds.Tables.Count > 0)
			{
				return ds.Tables[0].Rows.Count;
			}
			return 0;
		}
		#endregion
		#region QUERY

		private class BuildQuery
		{
			/// <summary>
			/// Gets the information of sscc
			/// </summary>
			/// <param name="sscc">The SSCC</param>      
			/// <returns></returns>
			public static string GetSSCCInformation(string SSCC)
			{
				StringBuilder strBuilder = new StringBuilder();
				strBuilder.Append("SELECT ");

				strBuilder.Append("COALESCE(SUM(");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.Quantity.NAME);
				strBuilder.Append("),0) AS ");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryDetailTable.Columns.Quantity.NAME));
				strBuilder.Append(", COUNT(DISTINCT ");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryDetailTable.Columns.ItemCode.NAME));
				strBuilder.Append(") AS ");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryDetailTable.Columns.ItemCode.NAME));

				strBuilder.Append(" FROM ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryDetailTable.NAME));

				strBuilder.Append(" WHERE ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.SSCC.NAME);
				strBuilder.Append(" = ");
				strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SSCC, CultureInfo.InvariantCulture));



				return strBuilder.ToString();
			}
		}

		#endregion
	}
}
